"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var operators_1 = require("rxjs/operators");
function extractRoute(parent, routeProperty, inherit) {
    if (inherit === void 0) { inherit = false; }
    if (inherit) {
        while (parent.firstChild) {
            parent = parent.firstChild;
        }
    }
    return parent[routeProperty];
}
function extractValues(args, stream$) {
    return stream$.pipe(operators_1.map(function (routeValues) {
        var values = args.reduce(function (data, arg) {
            if (routeValues && routeValues[arg]) {
                data[arg] = routeValues[arg];
            }
            return data;
        }, {});
        return args.length === 1 ? values[args[0]] : values;
    }));
}
function routeDecoratorFactory(routeProperty, args, extractor) {
    var config = (typeof args[args.length - 1] === "object" ? args.pop() : {});
    return function (prototype, key) {
        if (!args.length) {
            args = [key.replace(/\$$/, "")];
        }
        if (!prototype.ngOnInit) {
            throw (new Error(prototype.constructor.name + " uses the " + routeProperty + " @decorator without implementing 'ngOnInit'"));
        }
        var ngOnInit = prototype.ngOnInit;
        prototype.ngOnInit = function () {
            var _this = this;
            if (!this.route) {
                throw (new Error(this.constructor.name + " uses a route-xxl @decorator without a 'route: ActivatedRoute' property"));
            }
            var stream$ = extractor(this.route, routeProperty, config.inherit);
            stream$ = extractValues(args, stream$);
            if (config.pipe) {
                stream$ = stream$.pipe.apply(stream$, config.pipe);
            }
            if (config.observable === false) {
                stream$.subscribe(function (data) {
                    _this[key] = data;
                });
            }
            else {
                this[key] = stream$;
            }
            ngOnInit.call(this);
        };
    };
}
function RouteData() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory("data", args, extractRoute);
}
exports.RouteData = RouteData;
function RouteParams() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory("params", args, extractRoute);
}
exports.RouteParams = RouteParams;
function RouteQueryParams() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return routeDecoratorFactory("queryParams", args, function (route) { return route.queryParams; });
}
exports.RouteQueryParams = RouteQueryParams;
